# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2012 Thinkopen Solutions, Lda. All Rights Reserved
#    http://www.thinkopensolutions.com.
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
##############################################################################

from osv import osv, fields
from datetime import date
import time
from tools.translate import _

class account_move_sequence_regenarator(osv.osv):
    _name='account.move.sequence.regenerator'
    _description = 'Account Move Sequence Regenarator'
    
    def action_update_account_move_sequence(self, cr, uid, ids, context=None):
        print('Running...')
        
        obj = self.browse(cr, uid, ids[0], context=context)
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        
        account_move = self.pool.get('account.move')
        account_move_ids = account_move.search(cr, uid, [('company_id','=',user.company_id.id),('journal_id','=',obj.journal_id.id),('period_id','=',obj.period_id.id)], order='date asc')
        account_move_obj = account_move.browse(cr, uid, account_move_ids, context=context)
        
        account_move_line = self.pool.get('account.move.line')
        ir_sequence = self.pool.get('ir.sequence')
        
        if not account_move_ids:
            raise osv.except_osv(_('Warning!'), _("No account moves found!"))
        
        index = 1
        sequences_generated = []
        current_index_dict = {}
        for move in account_move_obj:
            print('#####################################################')
            print('Processing move %s of %s' % (index, str(len(account_move_obj))))
            index += 1
            print('Processing account move with id: %s' % str(move.id))
            
            
            #processing next_val hardcoded until account_move.code is not corrected as unique
            ir_sequence_obj = ir_sequence.browse(cr, uid, move.journal_id.sequence_id.id, context=context)
            
            #I guess the sequence_id will always be the same, but I'll check it anyway
            if current_index_dict.has_key(move.journal_id.sequence_id.id):
                current_index_dict[move.journal_id.sequence_id.id] = int(current_index_dict[move.journal_id.sequence_id.id]) + 1
            else:
                current_index_dict[move.journal_id.sequence_id.id] = 1
            
            args = { 'move_date' :  move.date}
            number = self._process(ir_sequence_obj.prefix, args) + '%%0%sd' % ir_sequence_obj.padding % current_index_dict[move.journal_id.sequence_id.id] + self._process(ir_sequence_obj.suffix, args)
            
            #to prevent number and reference confusion
            number = ('%s%s' % ('REF/',number))
            
            sequences_generated.append(number)
            
            if number:
                print('Updating move ref...')
                account_move.write(cr, uid, [move.id], { 'ref' :  number})
                account_move_line_ids = account_move_line.search(cr, uid, [('move_id','=',move.id)], order='date asc')
                print('Updating %d move line(s) with id(s): %s' % (len(account_move_line_ids), str(account_move_line_ids)))
                account_move_line.write(cr, uid, account_move_line_ids, { 'ref' :  number})
                print('\033[32;1m%s\033[0m' % ('Updated reference from %s to %s' % (str(move.ref), str(number))))
                
            else:
                print('\033[31m%s\033[0m' % 'Failed to update reference')
                
        print('#####################################################')
        print('References generated: %s' % str(sequences_generated).replace('u', ''))
        print('Finished!')
        return {'type': 'ir.actions.act_window_close'}
    
    #had do change the method to parse the move_date and not the current date (like in_sequence.def_process) 
    def _process(self, s, args={}):
        year, month, day = args['move_date'].split("-")
        start_period = date(int(year), int(month), int(day))
        return (s or '') % {
            'year':start_period.strftime('%Y'),
            'month': start_period.strftime('%m'),
            'day':start_period.strftime('%d'),
            'y': start_period.strftime('%y'),
            'doy': start_period.strftime('%j'),
            'woy': start_period.strftime('%W'),
            'weekday': start_period.strftime('%w'),
            'h24': start_period.strftime('%H'),
            'h12': start_period.strftime('%I'),
            'min': start_period.strftime('%M'),
            'sec': start_period.strftime('%S'),
        }
    
    def _get_journal(self, cr, uid, context=None):
        if context is None:
            context = {}
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        company_id = context.get('company_id', user.company_id.id)
        journal_obj = self.pool.get('account.journal')
        res = journal_obj.search(cr, uid, [('company_id', '=', company_id)], order='name')
        return res and res[0] or False
    
    def _get_period(self, cr, uid, context=None):
        if context is None:
            context = {}
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        company_id = context.get('company_id', user.company_id.id)
        period_obj = self.pool.get('account.period')
        res = period_obj.search(cr, uid, [('company_id', '=', company_id)],order='name')
        return res and res[0] or False
    
            
    _columns = {
                'journal_id': fields.many2one('account.journal', 'Journal', required=True),
                'period_id': fields.many2one('account.period', 'Period', required=True),
                }
    
    _defaults = {
                 'journal_id': _get_journal,
                 'period_id': _get_period,
                }

account_move_sequence_regenarator()